# Setup for Android

## Download the APK (Android Package)

Retrieve the APK file. There are many locations for this, but the main requirement is that it's patched to work with the server we'll be setting up. Try getting it at the locations below:

- [https://cdn.sif.moe/lovelive-community.apk](https://cdn.sif.moe/lovelive-community.apk)
- [https://ethanthesleepy.one/public/lovelive/sif/lovelive-community.apk](https://ethanthesleepy.one/public/lovelive/sif/lovelive-community.apk)

The apk has been modified to do the following:

- Automatic in-game language selection between English and Japanese based on system language or Android 13 per-app language setting.
- Runs natively in Android emulator. All binaries supported by Android, including x86-64, is available.
- Activates debug mode which show a little bit more information in-game.
- Backtrace, AppsFlyer, and Google Play is neutralized.
- Backtrace error is printed to logcat.
- Uses RSA private key from tungnotpunk/LLSIF@Home which should be compatible with LLSIF@Home and NPPS4.
- AppAssets.zip is patched to load from localhost:51376. This means it's compatible with LLSIF@Home and NPPS4.

This apk is signed with the following cert hash. This can be verified with an app like appverifier.
`07:31:46:BE:D1:B2:13:E6:00:57:DA:4C:05:4F:A1:9C:41:AA:99:77:E2:A8:E8:26:93:CB:84:54:E6:C1:8E:71`

Note: It does NOT contain patch to neutralize X-Message-Sign check. If you can't get XMS verification working, you need to patch it yourself. You should make your private server generate the correct XMS, though.
Note2: It's recommended to launch the game in English for the first time. Otherwise, when you launch the game in Japanese for the first time, switching to English will erase all your game data (excluding your account). This logic is in the Lua script of the game. Help on neutralizing this behavior is welcome.

How to generate X-Message-Sign:
- [Python](https://github.com/DarkEnergyProcessor/NPPS4/blob/b24b4e2a71be3c268c069a49bd3fae2bb25d7feb/npps4/util.py#L37-L42)
- [NodeJS](https://codeberg.org/dic1911/LLSIF.local/src/commit/e7e780258dca86c2f1a77577897fad23298fe9c7/handler/util.js#L18-L34)
- [Rust](https://github.com/ethanaobrien/sif-server/blob/4dbd8d837ebb1d497420359c340541e8c8c69316/src/router/global.rs#L73-L92)

After the APK is retrieved and installed, then you should install the private server.

## Modifying the app to point to your PC

Please see [APK patching](apk-patching.md).

## Seting up the private server

There are currently 2 recommended private servers for SIF:

- NPPS4: Written in Python. This server implements progression and attempts to emulate the official server as closely as possible. Gacha works.
- LLSIF@Home: Written in JavaScript using NodeJS. This server gives you all cards, and all songs are unlocked. There is no progression, and gacha does not work. **This method is recommended for those that are less-techsavvy.**

## Setting up LLSIF@Home
1. Install Termux
2. Run this command(s) inside of `Termux`:
```
curl http://ll.sif.moe/android | bash
```
***OR*** (if the command above fails do the commands below)
```
wget http://ll.sif.moe/android
bash android
```
(curl is broken currently, this will be one command if curl is fixed)
3. After it is done and it shows you the link to the discord/matrix server. **Enter `exit` and re-launch termux.**
4. You can run `runserver` to start the private server.
4. After this is done, you should be able to launch the game and it should just work. Message in the discord or matrix if help is needed.

## Setting up NPPS4
- Please refer to [this guide](https://gist.github.com/MikuAuahDark/eabf54e9db3f5004e3ccd9a72f0d79d5)
